==> openscada-0.6.2-2.tar.gz: openscada-0.6.2/src/resalloc.cpp <==

//OpenSCADA system file: resalloc.cpp
/***************************************************************************
 *   Copyright (C) 2003-2008 by Roman Savochenko                           *
 *   rom_as@fromru.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License.               *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tsys.h"
#include "resalloc.h"


//********************************************
//* Resource object                          *
//********************************************
Res::Res( unsigned val ) : rd_c(0)
{
    if( sem_init(&sem,0,val) || sem_init(&sem_rc,0,1) )
	throw TError("ResAlloc",_("Error open semaphor!"));
}

Res::~Res( )
{
    sem_wait( &sem );
    while( rd_c ) usleep(STD_WAIT_DELAY*1000);
    sem_destroy( &sem );
    sem_destroy( &sem_rc );
}

void Res::resRequestW( long tm )
{
    if( !tm ) sem_wait( &sem );
    else
    {
	timespec wtm = { tm, 0 };
	if( sem_timedwait( &sem, &wtm ) ) throw TError("ResAlloc",_("Timeouted!"));
    }
    //- Wait of readers free -
    if( rd_c )
    {
	time_t st_tm = time(NULL);
	while( rd_c )
	{
	    if( tm && st_tm+tm > time(NULL) ) throw TError("ResAlloc",_("Timeouted!"));
	    usleep(STD_WAIT_DELAY*1000);
	}
    }
}

void Res::resReleaseW( )
{
    sem_post( &sem );
}

void Res::resRequestR( long tm )
{
    if( !tm ) sem_wait( &sem );
    else
    {
	timespec wtm = { tm, 0 };
	if( sem_timedwait( &sem, &wtm ) ) throw TError("ResAlloc",_("Timeouted!"));
    }
    sem_wait( &sem_rc );
    rd_c++;
    sem_post( &sem_rc );

    sem_post( &sem );
}

void Res::resReleaseR( )
{
    sem_wait( &sem_rc );
    if( rd_c > 0 ) rd_c--;
    sem_post( &sem_rc );
}

//********************************************
//* Automatic resource allocator/deallocator *
//********************************************
ResAlloc::ResAlloc( Res &rid ) : m_id(rid), m_wr(0)
{

}

ResAlloc::ResAlloc( Res &rid, bool write, long tm ) : m_id(rid), m_wr(0)
{
    request( write, tm );
}

ResAlloc::~ResAlloc( )
{
    if( m_wr&0x01 ) release();
}

void ResAlloc::request( bool write, long tm )
{
    if( m_wr&0x01 ) release();
    m_wr |= 0x01;
    if( write ) 
    {
	m_wr |= 0x02;
	m_id.resRequestW(tm);
    }
    else m_id.resRequestR(tm);
}

void ResAlloc::release()
{
    if( !(m_wr&0x01) )	return;
    if( m_wr&0x02 ) m_id.resReleaseW( );
    else            m_id.resReleaseR( );
    m_wr &= ~0x03;
}

==> openscada-0.6.3.3.tar.gz: openscada-0.6.3.3/src/resalloc.cpp <==

//OpenSCADA system file: resalloc.cpp
/***************************************************************************
 *   Copyright (C) 2003-2008 by Roman Savochenko                           *
 *   rom_as@fromru.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License.               *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tsys.h"
#include "resalloc.h"

//********************************************
//* Resource object                          *
//********************************************
Res::Res( )
{
    if( pthread_rwlock_init(&rwc,NULL) )
	throw TError("ResAlloc",_("Error open semaphor!"));
}

Res::~Res( )
{
    pthread_rwlock_wrlock(&rwc);
    pthread_rwlock_destroy(&rwc);
}

void Res::resRequestW( long tm )
{
    if( !tm ) pthread_rwlock_wrlock(&rwc);
    else
    {
	timespec wtm;
	clock_gettime(CLOCK_REALTIME,&wtm);
	wtm.tv_sec += tm;
	if( pthread_rwlock_timedwrlock(CLOCK_REALTIME,&wtm) )
	    throw TError("ResAlloc",_("Timeouted!"));
    }
}

void Res::resRequestR( long tm )
{
    if( !tm ) pthread_rwlock_rdlock(&rwc);
    else
    {
	timespec wtm;
	clock_gettime(CLOCK_REALTIME,&wtm);
	wtm.tv_sec += tm;
	if( pthread_rwlock_timedrdlock(CLOCK_REALTIME,&wtm) )
	    throw TError("ResAlloc",_("Timeouted!"));
    }
}

void Res::resRelease( )
{
    pthread_rwlock_unlock(&rwc);
}

//********************************************
//* Automatic resource allocator/deallocator *
//********************************************
ResAlloc::ResAlloc( Res &rid ) : mId(rid), mAlloc(false)
{

}

ResAlloc::ResAlloc( Res &rid, bool write, long tm ) : mId(rid), mAlloc(false)
{
    request( write, tm );
}

ResAlloc::~ResAlloc( )
{
    if( mAlloc ) release();
}

void ResAlloc::request( bool write, long tm )
{
    if( mAlloc ) release();
    mAlloc = false;
    if( write ) mId.resRequestW(tm);
    else mId.resRequestR(tm);
    mAlloc = true;
}

void ResAlloc::release()
{
    if( !mAlloc ) return;
    mId.resRelease( );
    mAlloc = false;
}
